import { HTMLAttributes, useCallback, useMemo } from 'react';
import { FC } from 'react';
import { DisplayConfig } from '../models/display-config';
import { LedStrip } from './led-strip';
import tw, { css, styled, theme } from 'twin.macro';
import { ScreenshotDto } from '../models/screenshot.dto';
import { LedStripEditor } from './led-strip-editor';
import { LedStripConfig } from '../models/led-strip-config';

export interface DisplayWithLedStripsProps
  extends Omit<HTMLAttributes<HTMLElement>, 'onChange'> {
  config: DisplayConfig;
  screenshot: ScreenshotDto;
  onChange?: (config: DisplayConfig) => void;
}

const StyledContainer = styled.section(
  tw`m-4 grid gap-1`,
  css`
    grid-template-columns: ${theme`width.5`} ${theme`width.3`} auto ${theme`width.3`} ${theme`width.5`};
  `,
  css`
    grid-template-rows: ${theme`width.5`} ${theme`width.3`} auto ${theme`width.3`} ${theme`width.5`};
  `,
);

export const DisplayWithLedStrips: FC<DisplayWithLedStripsProps> = ({
  config,
  screenshot,
  onChange,
  ...htmlAttrs
}) => {
  const screenshotUrl = useMemo(
    () => `data:image/png;base64,${screenshot.encode_image}`,
    [screenshot.encode_image],
  );

  const onLedStripConfigChange = useCallback(
    (
      position:
        | 'top_led_strip'
        | 'left_led_strip'
        | 'right_led_strip'
        | 'bottom_led_strip',
      value: LedStripConfig | null,
    ) => {
      const c = { ...config, [position]: value };
      onChange?.(c);
    },
    [config],
  );
  return (
    <StyledContainer {...htmlAttrs}>
      <img src={screenshotUrl} tw="row-start-3 col-start-3" />
      <LedStrip
        config={config.top_led_strip}
        colors={screenshot.colors.top}
        tw="row-start-2 col-start-3"
      />
      <LedStrip
        config={config.left_led_strip}
        colors={screenshot.colors.left}
        tw="row-start-3 col-start-2"
      />
      <LedStrip
        config={config.right_led_strip}
        colors={screenshot.colors.right}
        tw="row-start-3 col-start-4"
      />
      <LedStrip
        config={config.bottom_led_strip}
        colors={screenshot.colors.bottom}
        tw="row-start-4 col-start-3"
      />
      <LedStripEditor
        config={config.top_led_strip}
        tw="row-start-1 col-start-3"
        onChange={(value) => onLedStripConfigChange('top_led_strip', value)}
      />
      <LedStripEditor
        config={config.left_led_strip}
        tw="row-start-3 col-start-1"
        onChange={(value) => onLedStripConfigChange('left_led_strip', value)}
      />
      <LedStripEditor
        config={config.right_led_strip}
        tw="row-start-3 col-start-5"
        onChange={(value) => onLedStripConfigChange('right_led_strip', value)}
      />
      <LedStripEditor
        config={config.bottom_led_strip}
        tw="row-start-5 col-start-3"
        onChange={(value) => onLedStripConfigChange('bottom_led_strip', value)}
      />
    </StyledContainer>
  );
};
