import { invoke } from '@tauri-apps/api';
import { FC, Fragment, useCallback, useEffect, useMemo, useState } from 'react';
import tw from 'twin.macro';
import { useAsync, useAsyncCallback } from 'react-async-hook';
import { DisplayWithLedStrips } from './components/display-with-led-strips';
import { PickerConfiguration } from './models/picker-configuration';
import { DisplayConfig } from './models/display-config';
import { ScreenshotDto } from './models/screenshot.dto';
import { Alert, Snackbar } from '@mui/material';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { faSpinner } from '@fortawesome/free-solid-svg-icons';
import { update } from 'ramda';

const getPickerConfig = () => invoke<PickerConfiguration>('get_picker_config');
const getScreenshotOfDisplays = () =>
  invoke<ScreenshotDto[]>('take_snapshot').then((items) => {
    console.log(items);
    return items;
  });
const getScreenshotByConfig = async (config: DisplayConfig) => {
  return await invoke<ScreenshotDto>('get_screenshot_by_config', {
    config,
  });
};

export const Configurator: FC = () => {
  const { loading: pendingPickerConfig, result: savedPickerConfig } = useAsync(
    getPickerConfig,
    [],
  );

  const { loading: pendingScreenshotOfDisplays, result: defaultScreenshotOfDisplays } =
    useAsync(getScreenshotOfDisplays, []);

  const [screenshotOfDisplays, setScreenshotOfDisplays] = useState<ScreenshotDto[]>([]);

  const { loading: pendingGetLedColorsByConfig, execute: onPickerChange } =
    useAsyncCallback(async (value: DisplayConfig) => {
      console.log(value);
      const screenshot = await getScreenshotByConfig(value);
      setScreenshotOfDisplays((old) => {
        const index = old.findIndex((it) => it.config.id === screenshot.config.id);
        console.log({ old, n: update(index, screenshot, old) });
        return update(index, screenshot, old);
      });

      console.log('screenshot', screenshot);
    });

  const [displayConfigs, setDisplayConfigs] = useState<DisplayConfig[]>([]);

  useEffect(() => {
    const displayConfigs = savedPickerConfig?.display_configs;
    if (displayConfigs) {
      setDisplayConfigs(displayConfigs);
    }
  }, [savedPickerConfig]);
  useEffect(() => {
    if (defaultScreenshotOfDisplays) {
      setScreenshotOfDisplays(defaultScreenshotOfDisplays);
    }
  }, [defaultScreenshotOfDisplays]);

  const displays = useMemo(() => {
    if (screenshotOfDisplays) {
      console.log({ c: screenshotOfDisplays });
      return screenshotOfDisplays.map((screenshot, index) => (
        <DisplayWithLedStrips
          key={index}
          config={screenshot.config}
          screenshot={screenshot}
          onChange={(value) => onPickerChange(value)}
        />
      ));
    }
  }, [displayConfigs, screenshotOfDisplays]);

  if (pendingPickerConfig || pendingScreenshotOfDisplays) {
    return (
      <section>
        等待 {JSON.stringify({ pendingPickerConfig, pendingScreenshotOfDisplays })}
      </section>
    );
  }

  return (
    <Fragment>
      <section>{displays}</section>;
      <Snackbar open={pendingGetLedColorsByConfig} autoHideDuration={3000}>
        <Alert icon={<FontAwesomeIcon icon={faSpinner} />} sx={{ width: '100%' }}>
          This is a success message!
        </Alert>
      </Snackbar>
    </Fragment>
  );
};
